/* 
Read in the area-level dataset:
County = unique area identifier.
y = measure of wind erosion in Iowa counties.
Erodibility = the only auxiliary variable.
Samvar = sampling variances, assumed known.
*/
libname SAE101 "H:\ClassesTaught\SAE101\Jan2013";

* Look at the y-values and their CVs;
* Some are above 0.20, so maybe they can be reduced;
data temp;
  set sae101.erosion;
  direct_cv = sqrt(samvar)/y;
run;
proc print data=temp noobs;
  var county y direct_cv ;
  format y 6.2 direct_cv 6.2;
run;

* Look at a linear regression to see if model is sensible;
* and compare y-values with predicted values;
ods graphics on;
proc reg data=SAE101.erosion;
  model y = Erodibility / mse ;
  output out=work.tempout predicted=LSmeans;
run;quit;
ods graphics off;
data temp;
  merge temp tempout;
run;
proc print data=temp noobs;
  var county y LSmeans direct_cv ;
  format y 6.2 LSmeans 6.2 direct_cv 6.2;
run;
* Note that even some of the low-CV direct estimates
  differ a lot from their regression predictions;



/* 
Estimate the model error variance
using Prasad-Rao estimator:
Run regression to get root MSE and square it,
  which is (sum of squared resids / m-p)
Also store hat matrix diagonals
  and combine with samvars
  to get sum(v*(1-h))/(m-p)
Subtract the two, take max of that or 0,
  and we have our estimator.
*/

* Re-run the unweighted least squares regression
  but store the hat matrix diagonals this time;
* Also look at plots (residuals, etc) to confirm if model is sensible;
ods graphics on;
proc reg data=SAE101.erosion outest=work.tempoutest;
  model y = Erodibility / mse ;
  output out=work.tempout h=hatdiag;
run;quit;
ods graphics off;
* Store the MSE for the modelvar-based component,
  and the error degrees of freedom (m-p);
data work.msecomp(keep=msecomp _edf_);
  set work.tempoutest(rename=(_mse_=msecomp));
run;
* Compute each samvar-based component,
  v_i * (1-r_i);
* Only use the non-missing-y data that were used for the regression;
data work.tempout;
  set work.tempout;
  if y ne .;
  samvarcomp = samvar*(1-hatdiag);
run;
* Sum up all the samvar-based components;
proc means data=work.tempout noprint;
  output out=work.samvarcomp(keep=samvarcomp) sum(samvarcomp)=;
run;
* Compute the final estimate of model error variance;
* Remember to divide the sum of samvarcomps by (m-p) before subtracting;
* And set it to 0 if the estimate is negative;
data work.modvarest(keep=modvarest);
  merge work.msecomp work.samvarcomp;
  modvarest=max(0, msecomp-(samvarcomp/_edf_));
run;


/*
Now that we have model error variance,
  use it to create regression weights
  and run WLS regression to get estimates of X*Beta.
Then build the shrinkage weights to get the shrinkage estimators.
Finally estimate MSEs and CVs.
*/

* Merge modvarest in with the other data;
* And sum the two variance components for each observation;
data work.datawithmodvar;
  if _N_=1 then set work.modvarest;
  set sae101.erosion;
  totvar = modvarest+samvar;
  totvarinv = 1/totvar;
run;
* Run WEIGHTED least squares regression;
proc reg data=work.datawithmodvar;
  model y = Erodibility / noprint;
  weight totvarinv;
  output out=work.WLSout predicted=WLSmeans stdp=XB_stderr;
run;quit;
* Create the shrinkage weights;
* If shrinkwt=1, use the synthetic estimate only;
* As shrinkwt goes to 0, rely more on the direct estimate;
* Then shrink together the direct estimates and regression means;
* And estimate the MSEs and CVs;
data work.temp(rename=(y=direct_est));
  set work.WLSout;
  if (samvar=. | y=.) then do;
	shrinkwt=1;
	SAE_est = WLSmeans;
	g1 = modvarest;
	g2 = XB_stderr**2;
  end;
  else do;
    shrinkwt = samvar/totvar;
    SAE_est = (1-shrinkwt)*y + shrinkwt*WLSmeans;
	g1 = shrinkwt*modvarest;
	g2 = (shrinkwt**2)*(XB_stderr**2);
  end;
  SAE_mse = g1+g2;
  SAE_stderr = sqrt(SAE_mse);
  SAE_CV = SAE_stderr/SAE_est;
  DIRECT_stderr = sqrt(samvar);
  DIRECT_cv = DIRECT_stderr/y;
run;
* Print the results nicely;
proc print data=temp noobs;
  var county erodibility direct_est sae_est direct_stderr sae_stderr direct_cv sae_cv;
  format erodibility 6.2 direct_est 6.2 sae_est 6.2 
		direct_stderr 6.2 sae_stderr 6.2 direct_cv 6.2 sae_cv 6.2;
run;

