* First, run SAE_AreaLevel_prep.sas
  to read in the area-level dataset:
County = unique area identifier.
y = measure of wind erosion in Iowa counties.
Erodibility = the only auxiliary variable.
Samvar = sampling variances, assumed known;
libname SAE101 "H:\ClassesTaught\SAE101\Jan2013";

* Drop the rows with missing y-values for now,
  since I was unable to get PROC MCMC to impute them;
data SAE101.erosionNoMiss;
  set SAE101.erosion;
  if y ne .;
run;

* Add an index number variable to each observation in the dataset,
  and create a macro variable to record the total number of areas;
data SAE101.erosionNoMiss;
  set SAE101.erosionNoMiss end=eof;
  ind = _N_;
  if eof then call symput('NrAreas',_N_);
run;

********** PROC MCMC STARTS **********;

* Run a Bayesian model with PROC MCMC,
	and store the results.
  Let y be the direct survey estimate, and mu be the unknown true value.
  mu1 through mu44 are the target values for the 44 complete-data counties.
  b0 and b1 are intercept and slope of the regression of y vs Erodibility.
  sigmam is the model error standard deviation (not variance!)
  of mu around the regression line XB;
* We will explicitly model the model errors as theta, that is,
    mu = b0 + x1*b1 + theta
    theta ~ Normal(0, sd=sigmam)
  Ideally we would just put a prior directly on mu
    mu ~ Normal(b0 + x1*b1, sd=sigmam)
  but that second approach is tricky in SAS.
  See SAE_AreaLevel_ProcMCMC_alt.sas for an example;
* The approach here is based on
  Mixed Effects Model example from SAS 9.2 docs
  http://support.sas.com/documentation/cdl/en/statug/63033/HTML/default/viewer.htm#statug_mcmc_sect006.htm
  along with
  Assignments Of Parameters in SAS 9.2 docs
  http://support.sas.com/documentation/cdl/en/statug/63033/HTML/default/viewer.htm#statug_mcmc_sect024.htm
;
ods graphics on;
ods output PostSummaries = work.PostSum PostIntervals = work.PostInt;
proc mcmc data=SAE101.erosionNoMiss thin=100 nmc=100000
	stats(alpha=0.1) monitor=(b0 b1 sigmam mu);
	* We want to run 100000 iterations, and keep only every 100th of them;
	* We want 90% credible intervals, not the default 95%;
	* We want to monitor b0, b1, sigmam, and mu, but not theta;

	* Create array for the the model error terms theta;
	array theta[&NrAreas.];

	* We want to monitor the mu values so they also need an array
	  which will be indexed by the variable ind
	  so that we can match up array elements with dataset rows;
	array mu[&NrAreas.];

	* Initialize each theta at a different reasonable value,
	  not all thetas at the same constant value,
	  else SAS will think sigmam=0 is a good guess and get stuck there;
	begincnst;
		theta[ind] = y-(b0+b1*Erodibility);
	endcnst;

	* Declare the parameters that will have priors, and initialize;
	* The parameters that are declared in separate blocks
	  will be accepted/rejected independently of other parms blocks;
	parms theta: ;
	parms b0 0.8 b1 0.2;
	parms sigmam 0.16;

	* Define the prior distributions;
	prior theta: ~ normal(0, sd=sigmam);
	* Using improper flat priors on the b values (regression coefficients);
	prior b: ~ general(0);
	* Using flat prior on stdev, not on variance
	  and not the usual conjugate inverse-gamma prior.
	  See details in Gelman (2006) paper;
	prior sigmam ~ uniform(0,1000);

	* Define the linking model, mu = b0 + b1*x1 + theta;
	* Recall that the mu parameters we care about are in an array,
	  while the data y are in rows in the dataset,
	  so use ind to match up array elements with dataset rows;
	mu[ind] = b0 + b1*Erodibility + theta[ind];

	* Finally, define the model linking the data to parameters;
	model y ~ normal(mu[ind], var=samvar);
run;
ods graphics off;

********** PROC MCMC ENDS **********;

* Look at output to see our posterior estimate of sigmam is around 0.16
  and recall that this is a standard error, not a variance
  so it is consistent with the PROC MIXED model error variance estimate
  of around 0.025 which is close to 0.16^2;

* Look at output to check for convergence:
  Is MonteCarloStandardError a small fraction of the StandardDevation?
  Under the Diagnostic Plots, do the tracesseem like white noise?
  Do the Autocorrelations die off quickly? etc.;

* Merge the summary statistics together (posterior means, stdevs, and 90% intervals) 
  and drop the other parameters (b0, b1, sigmam) so that we keep only the mu values;
data work.MCMCResults (keep=mean stddev cv crediblelower credibleupper);
  merge work.PostSum work.PostInt;
  if substr(parameter,1,2)='mu';
  CV = stddev/mean;
run;

* Merge and print the results;
data SAE101.ErosionResults(drop=samvar);
  retain county erodibility direct_est mcmc_est direct_stderr mcmc_stderr direct_cv mcmc_cv mcmc_ci_lo mcmc_ci_hi;
  merge MCMCResults(rename=(mean=mcmc_est stddev=mcmc_stderr crediblelower=mcmc_ci_lo credibleupper=mcmc_ci_hi cv=mcmc_cv))
		sae101.erosion(rename=(y=direct_est));
  if _N_ le 44;
  direct_stderr=sqrt(samvar);
  direct_cv = direct_stderr/direct_est;
  label mcmc_est="MCMC Estimate"
		mcmc_stderr="MCMC Std Err";
run;
proc print data=SAE101.ErosionResults noobs;
  var county direct_est mcmc_est direct_stderr mcmc_stderr direct_cv mcmc_cv mcmc_ci_lo mcmc_ci_hi;
  format direct_est 6.2 mcmc_est 6.2 
		direct_stderr 6.2 mcmc_stderr 6.2 direct_cv 6.2 mcmc_cv 6.2
		mcmc_ci_lo 6.2 mcmc_ci_hi 6.2;
run;
