* Code to accompany SAE_AreaLevel_ProcMCMC.sas
  showing two alternate approaches:
  first, using a redundant parameterization (with more parameters),
  and second, using hierarchical centering (with fewer parameters)
  but requiring explicit definition of log-prior function.
  Both approaches might help speed up MCMC convergence;

*************************;

* Alternate approach using Gelman 2006 model (2) approach
  of breaking up theta into a product ksi*eta
  which is an equivalent model but may converge faster:
    mu = b0 + x1*b1 + ksi*eta
    eta ~ Normal(0, sd=sigmae)
    ksi ~ Normal(0, 1)
  and therefore in our old terminology
    theta = ksi*eta ~ Normal(0, sd=abs(ksi)*sigmae)
    sigmam = abs(ksi)*sigmae
;
ods graphics on;
ods output PostSummaries = work.PostSum PostIntervals = work.PostInt;
proc mcmc data=SAE101.erosionNoMiss thin=100 nmc=100000
	stats(alpha=0.1) monitor=(b0 b1 sigmam mu);

	array mu[&NrAreas.];

	* Define array for eta, not theta;
	array eta[&NrAreas.];
	* Use same initial values as before
	  since we will start ksi at 1;
	begincnst;
		eta[ind] = y-(b0+b1*Erodibility);
	endcnst;

	parms eta: ;
	parms b0 0.8 b1 0.2;
	parms sigmae 0.16;
	parms ksi 1;

	prior eta: ~ normal(0, sd=sigmae);
	prior b: ~ general(0);
	prior sigmae ~ uniform(0,1000);
	prior ksi ~ normal(0, sd=1);

	mu[ind] = b0 + b1*Erodibility + ksi*eta[ind];
	sigmam = abs(ksi)*sigmae;
	model y ~ normal(mu[ind], var=samvar);
run;
ods graphics off;

*************************;

* Alternate approach using hierarchical centering based on
  Random Effects Models example from SAS 9.2 docs
  http://support.sas.com/documentation/cdl/en/statug/63033/HTML/default/viewer.htm#statug_mcmc_sect047.htm
  Here we avoid the theta parameter altogether
  and we specify the prior for mu directly,
  though more awkwardly because it relies on the data x values
;
ods graphics on;
ods output PostSummaries = work.PostSum PostIntervals = work.PostInt;
proc mcmc data=SAE101.erosionNoMiss thin=100 nmc=100000
	stats(alpha=0.1);

* Create array of the mu parameters we want to estimate;
array mu[&NrAreas.];

* Set up the parameters with initial values,
  with variance param in a separate block
  so it can be accepted/rejected independently of other params;
parms b0 0 b1 0 mu: 1; * This initializes each mu to the same value;
parms sigmam 0.16;

* Set up loglikelihood for prior on the mu random effects
  (This is awkward in SAS 9.2. We must give log-prior explicitly.
  But it could be done more simply in SAS 9.3, using a RANDOM statement.);
* Recall that the mu parameters we care about are in an array,
  while the data y are in rows in the dataset;
* You must define log-prior (lp) before using it in the PRIOR statements below;
XB = b0 + b1*Erodibility;
if ind eq 1 then
	lp = lpdfnorm(mu[ind], XB, sigmam);
else
	lp = lp + lpdfnorm(mu[ind], XB, sigmam);

* Note: be careful about standard deviation vs variance:
    inputs to lpdfnorm() above are (data, mu, sd),
    while inputs to normal() below may be (mu, var) or (mu, sd);

* Define priors on all parameters;
*   Using flat prior on stdev, not on variance
	and not the usual conjugate inverse-gamma prior.
	See details in Gelman (2006) paper;
prior sigmam ~ uniform(0,1000);
*   And using improper flat prior on b (regression coefficients);
prior b: ~ general(0);
*   And using the log-lkhd defined above for mu;
prior mu: ~ general(lp);

* Define the model linking the data to parameters;
model y ~ normal(mu[ind], var=samvar);
run;
ods graphics off;
