#####
# R and WinBUGS example of the Fay-Herriot area-level model
# for Small Area Estimation workshop
# Jerzy Wieczorek, Jan 31, 2013

#####
# General setup

# Load the R2WinBUGS package
library(R2WinBUGS)

# Set options to print output with 2 significant digits
# (R's default gives too much detail)
options(digits=2)

# Set the working directory
setwd('H:/ClassesTaught/SAE101/Jan2013')

#####
# Define the model itself
# This is a version with hierarchical centering:
#   y ~ Normal(Y, samvar)
#   Y ~ Normal(b0 + x1*b1, sigma2.M)
# For the versions with theta or with ksi*eta,
# see the alternate examples at the bottom

# (Note that in WinBUGS, the Normal distribution parameters
#  are the mean and *precision* = 1/variance,
#  rather than using variance or standard deviation directly)
erosionmodel <- function(){
     for (i in 1:m){
       y[i] ~ dnorm (Y[i], tau.y[i])
       Y[i] ~ dnorm (XB[i], tau.M)
       tau.y[i] <- pow(samvar[i], -1)
       XB[i] <- b0 + x[i]*b1
     }
     b0 ~ dnorm (0.0, 1.0E-6)
     b1 ~ dnorm (0.0, 1.0E-6)
     sigma.M ~ dunif (0, 1000)
     tau.M <- pow(sigma.M, -2)
     sigma2.M <- pow(sigma.M, 2)
}
# Write the model to a file
write.model(erosionmodel, "erosionmodel.bug")

#####
# Prepare the data, initialize parameter values, etc

# Load in the dataset from a .csv file
erosion <- read.csv('erosion.csv')

# Define the data that WinBUGS will need
m <- nrow(erosion)
y <- erosion$y
x <- erosion$Erodibility
samvar <- erosion$Samvar

# List the names of the data we need to give WinBUGS
erosiondata <- list("m","y","x","samvar")

# Set a function to generate random initial values
# for the non-data random parameters
erosioninits <- function(){
    list(Y=rnorm(m, 0, 100),
         b0=rnorm(1, 0, 100), b1=rnorm(1, 0, 100),
         sigma.M=runif(1, 0, 100))
}

# Make a vector with names of parameters we want WinBUGS to track
# (it will not store the others, like tau.y or XB or sigma.M)
erosionparameters <- c("Y", "b0", "b1", "sigma2.M")

#####
# Run the model and analyze results

# Tell WinBUGS to run the model, with 3 chains
erosion.sim <- bugs(erosiondata, erosioninits,
	erosionparameters, "erosionmodel.bug",
	n.chains=3, n.iter=1000,
	bugs.directory="c:/Program Files/WinBUGS14/", debug=TRUE)

# Print and plot the results
print(erosion.sim)
plot(erosion.sim)

# Plot the posterior densities of the model parameters
plot(density(erosion.sim$sims.list$sigma2.M))
plot(density(erosion.sim$sims.list$b0))
plot(density(erosion.sim$sims.list$b1))

#####
# Prepare results for printing and saving

# Save and display the same results as in our SAS examples
Direct_Est <- y
SAE_Est <- erosion.sim$mean$Y
Direct_StdErr <- sqrt(samvar)
SAE_StdErr <- erosion.sim$sd$Y
Direct_CV <- Direct_StdErr/Direct_Est
SAE_CV <- SAE_StdErr/SAE_Est
ErosionResults <- cbind(County=erosion$County,
	Direct_Est, SAE_Est,
	Direct_StdErr, SAE_StdErr,
	Direct_CV, SAE_CV)
ErosionResults
# Should look very similar to the SAS output,
# but the SAE stderrs here account for variability in sigma2.M
# so they will be a little larger.

# To add the 90% confidence intervals,
# find the 5% and 95% quantiles of each estimate's MCMC chain
ErosionCIs <- t(apply(erosion.sim$sims.list$Y, 2, quantile, probs=c(.05,.95)))
colnames(ErosionCIs) <- c("SAE_CI_lo","SAE_CI_hi")
ErosionResults <- cbind(ErosionResults,ErosionCIs)
ErosionResults 

# Save only the SAE results (we wouldn't usually publish the direct estimates)
ErosionResultsToSave <- ErosionResults[,c("County","SAE_Est","SAE_StdErr","SAE_CV","SAE_CI_lo","SAE_CI_hi")]
write.csv(ErosionResultsToSave,"ErosionResults.csv", row.names=FALSE)

#######################################################
# Optional steps

#####
# Compare the direct estimates, regression predictions, and SAEs
XB <- erosion.sim$mean$b0 + x * erosion.sim$mean$b1
cbind(Direct_Est, SAE_Est, XB)

# Look at shrinkage weights to make sure
# that they are not all near 0 or all near 1
w <- samvar / (samvar + erosion.sim$mean$sigma2.M)
hist(w)


#######################################################
#######################################################
# Alternate models

#####
# 1st alternate model, without hierarchical centering
# (sometimes one version may converge faster than the other)
#   y ~ Normal(Y, samvar)
#   Y = b0 + x1*b1 + theta
#   theta = Normal(0, sigma2.M)
# For the other alternate version with ksi*eta,
# see alternate example 2 below
erosionmodel.alt1 <- function(){
     for (i in 1:m){
       y[i] ~ dnorm (Y[i], tau.y[i])
       Y[i] <- XB[i] + theta[i]
       theta[i] ~ dnorm (0, tau.M)
       tau.y[i] <- pow(samvar[i], -1)
       XB[i] <- b0 + x[i]*b1
     }
     b0 ~ dnorm (0.0, 1.0E-6)
     b1 ~ dnorm (0.0, 1.0E-6)
     sigma.M ~ dunif (0, 1000)
     tau.M <- pow(sigma.M, -2)
     sigma2.M <- pow(sigma.M, 2)
}
write.model(erosionmodel.alt1, "erosionmodel.alt1.bug")

# Also must tell it to initialize the thetas, not Ys
erosioninits.alt1 <- function(){
    list(theta=rnorm(m, 0, 1),
         b0=rnorm(1, 0, 100), b1=rnorm(1, 0, 100),
         sigma.M=runif(1, 0, 100))
}

# Tell WinBUGS to run the alternate model
erosion.sim.alt1 <- bugs(erosiondata, erosioninits.alt1,
	erosionparameters, "erosionmodel.alt1.bug",
	n.chains=3, n.iter=1000,
	bugs.directory="c:/Program Files/WinBUGS14/", debug=TRUE)

# Print and plot the results
print(erosion.sim.alt1)
plot(erosion.sim.alt1)


#####
# 2nd alternate model, without hierarchical centering
# but with redundant reparameterization
#   y ~ Normal(Y, samvar)
#   Y = b0 + x1*b1 + ksi*eta
#   eta ~ Normal(0, sigma2.eta)
#   ksi ~ Normal(0, 1)
#   theta = ksi*eta 
#   sigma2.M = abs(ksi)*sigma2.eta
# This is based on Gelman (2006) paper
# For the other alternate version with theta,
# see alternate example 1 above
erosionmodel.alt2 <- function(){
     for (i in 1:m){
       y[i] ~ dnorm (Y[i], tau.y[i])
       Y[i] <- XB[i] + theta[i]
       theta[i] <- ksi*eta[i]
       eta[i] ~ dnorm (0, tau.eta)
       tau.y[i] <- pow(samvar[i], -1)
       XB[i] <- b0 + x[i]*b1
     }
     b0 ~ dnorm (0.0, 1.0E-6)
     b1 ~ dnorm (0.0, 1.0E-6)
     ksi ~ dnorm (0.0, 1)
     sigma.eta ~ dunif (0, 1000)
     sigma.M <- abs(ksi)*sigma.eta
     tau.eta <- pow(sigma.eta, -2)
     sigma2.M <- pow(sigma.M, 2)
}
write.model(erosionmodel.alt2, "erosionmodel.alt2.bug")

# Also must tell it to initialize the etas and sigma.eta and ksi,
# not Ys or thetas and sigma.M
erosioninits.alt2 <- function(){
    list(eta=rnorm(m, 0, 1), ksi=rnorm(1, 0, 1),
         b0=rnorm(1, 0, 100), b1=rnorm(1, 0, 100),
         sigma.eta=runif(1, 0, 100))
}

# Tell WinBUGS to run the alternate model
erosion.sim.alt2 <- bugs(erosiondata, erosioninits.alt2,
	erosionparameters, "erosionmodel.alt2.bug",
	n.chains=3, n.iter=1000,
	bugs.directory="c:/Program Files/WinBUGS14/", debug=TRUE)

# Print and plot the results
print(erosion.sim.alt2)
plot(erosion.sim.alt2)

